//
// QPerforce is a gui interface to the perforce revision control
// system. It is based on the qt library, and should be easily portable
// across any platform that has this toolkit available.
// 
// Copyright (C) 2002 Jacob Gladish
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#include "qperforcewindow.h"

#include "BranchSpecMenu.h"
#include "ClientSpecMenu.h"
#include "EditMenu.h"
#include "FileMenu.h"
#include "HelpMenu.h"
#include "JobMenu.h"
#include "LabelMenu.h"
#include "SettingsMenu.h"
#include "StatusPane.h"
#include "ToolsMenu.h"
#include "UserMenu.h"
#include "ViewMenu.h"

#include <qfont.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmenubar.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qvbox.h>
#include <qwidgetstack.h>


const char* _widget_names [] = {
  "Perforce Users"
};




qperforceWindow* qperforceWindow::_instance = NULL;


qperforceWindow* qperforceWindow::getInstance() 
{
  if (_instance == NULL) {
    _instance = new qperforceWindow();
  }
  return _instance;

}


qperforceWindow::qperforceWindow( QWidget* parent, const char* name, WFlags f )
	: QMainWindow( parent, name, f )
{
	setCaption( "qperforce" );
  resize( QSize(800,600) );

  buildMenuBar();
	// buildToolBar();

  _verticalSplitter = new QSplitter( Qt::Vertical, this );    
  _topHorizontalSplitter = new QSplitter( Qt::Horizontal, _verticalSplitter );
    
  _statusPane = new StatusPane( _verticalSplitter );

  //======================================================================
  // setup the left and right panes with the proper layouts for the
  // titles
  //  
  // left box
  QFont titleFont( "Sans Serif", 10, QFont::Bold, false, QFont::ISO_8859_1 ); 
  QVBox* leftBox = new QVBox( _topHorizontalSplitter );
  _leftPaneTitle = new QLineEdit( leftBox );
  _leftPaneTitle->setReadOnly( true );
  _leftPaneTitle->setFont( titleFont );
  QLabel* left = new QLabel( "Top Left", leftBox );
  //
  // right box
  //
  QVBox* rightBox = new QVBox( _topHorizontalSplitter );
  _rightPaneTitle = new QLineEdit( rightBox );
  _rightPaneTitle->setReadOnly( true );
  _rightPaneTitle->setFont( titleFont );
  _rightWidgetStack = new QWidgetStack( rightBox );

  buildRightWidgetStack();      
  //
  // end layout
  //======================================================================


  //======================================================================
  // setup signals/slots
  //
  QObject::connect( _userMenu, SIGNAL(showUsers(WidgetId)),
                    this, SLOT(showWidget(WidgetId)) );  
  //======================================================================

  setCentralWidget( _verticalSplitter );
}


void qperforceWindow::buildMenuBar()
{
  _menuBar = new QMenuBar( this );
  _statusBar = new QStatusBar( this );
  
  _fileMenu = new FileMenu();  
  _menuBar->insertItem( "&File", _fileMenu );

  _editMenu = new EditMenu();  
  _menuBar->insertItem( "&Edit", _editMenu );

  _viewMenu = new ViewMenu();  
  _menuBar->insertItem( "&View", _viewMenu );

  _settingsMenu = new SettingsMenu();  
  _menuBar->insertItem( "&Settings", _settingsMenu );

  _branchSpecMenu = new BranchSpecMenu();  
  _menuBar->insertItem( "&BranchSpec", _branchSpecMenu );

  _labelMenu = new LabelMenu();  
  _menuBar->insertItem( "&Label", _labelMenu );

  _clientSpecMenu = new ClientSpecMenu();  
  _menuBar->insertItem( "&ClientSpec", _clientSpecMenu );

  _userMenu = new UserMenu();  
  _menuBar->insertItem( "&User", _userMenu );

  _jobMenu = new JobMenu();  
  _menuBar->insertItem( "&Job", _jobMenu );

  _toolsMenu = new ToolsMenu();  
  _menuBar->insertItem( "&Tools", _toolsMenu );

  _helpMenu = new HelpMenu();
  _menuBar->insertItem( "Help", _helpMenu );

  _viewMenu->connectItem( _viewMenu->getViewUsersId(), 
    _userMenu, SLOT(slotViewUsers()) );

}

void qperforceWindow::buildToolBar()
{
  _toolBar = new QToolBar( this, "MainToolBar" );
  CHECK_PTR( _toolBar );
}



void qperforceWindow::setStatusMessage(const QString &m)
{
  ASSERT( _statusBar != NULL );
  _statusBar->message( m );
}

void qperforceWindow::showStatusBar( bool b )
{
  if (_statusBar != NULL) {
    if (b) {
      _statusBar->show();
    } else {
      _statusBar->hide();
    }
  }
}

void qperforceWindow::appendUserMessage( const QString& s )
{
  ASSERT( _statusPane != NULL );
  _statusPane->append( s );
}


void qperforceWindow::buildRightWidgetStack()
{
  _userTable = new UserTable( _rightWidgetStack );
  _userMenu->setListView( _userTable );

  _rightWidgetStack->addWidget( _userTable, USERS_TABLE );
  
}


void qperforceWindow::showWidget( WidgetId id )
{  
  _rightWidgetStack->raiseWidget( id );  
  _rightPaneTitle->setText( _widget_names[id] );
}

